@@ -5,31 +5,39 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import __builtin__
-
 import gettext
 import locale
 import os
 import sys
 import warnings
 
-import quodlibet.const
-import quodlibet.util
+from quodlibet.compat import builtins, PY2
 
-from quodlibet.util import set_process_title
+if PY2:
+    # some code depends on utf-8 default encoding (pygtk used to set it)
+    reload(sys)
+    sys.setdefaultencoding("utf-8")
+
+from quodlibet.util import set_process_title, environ, cached_func
+from quodlibet.util import windows, is_osx, is_windows
 from quodlibet.util.path import mkdir, unexpand
 from quodlibet.util.i18n import GlibTranslations, set_i18n_envvars, \
     fixup_i18n_envvars
 from quodlibet.util.dprint import print_, print_d, print_w, print_e
 from quodlibet import const
-from quodlibet.const import MinVersions, Version
+from quodlibet.const import MinVersions
+from quodlibet.compat import PY2
+
 
 PLUGIN_DIRS = ["editing", "events", "playorder", "songsmenu", "playlist",
-               "gstreamer", "covers"]
+               "gstreamer", "covers", "query"]
 
 
 GlibTranslations().install(unicode=True)
 
+_cli_initialized = False
+_initialized = False
+
 
 class Application(object):
     """A main application class for controlling the application as a whole
@@ -48,6 +56,9 @@ class Application(object):
     library = None
     player = None
 
+    player_options = None
+    """A PlayerOptions instance or None in case there is no playback support"""
+
     cover_manager = None
 
     name = None
@@ -56,6 +67,14 @@ class Application(object):
     id = None
     """The application ID e.g. 'quodlibet'"""
 
+    @property
+    def icon_name(self):
+        return self.id
+
+    @property
+    def symbolic_icon_name(self):
+        return "%s-symbolic" % self.icon_name
+
     @property
     def librarian(self):
         return self.library.librarian
@@ -103,6 +122,62 @@ def is_release():
     return const.VERSION_TUPLE[-1] != -1
 
 
+@cached_func
+def get_base_dir():
+    """The path to the quodlibet package"""
+
+    file_path = __file__
+    if os.name == "nt":
+        file_path = file_path.decode(sys.getfilesystemencoding())
+    return os.path.dirname(os.path.realpath(file_path))
+
+
+@cached_func
+def get_image_dir():
+    """The path to the image directory in the quodlibet package"""
+
+    return os.path.join(get_base_dir(), "images")
+
+
+@cached_func
+def get_user_dir():
+    """Place where QL saves its state, database, config etc."""
+
+    if os.name == "nt":
+        USERDIR = os.path.join(windows.get_appdate_dir(), "Quod Libet")
+    else:
+        USERDIR = os.path.join(os.path.expanduser("~"), ".quodlibet")
+
+    if not PY2:
+        USERDIR += "_py3"
+
+    if 'QUODLIBET_USERDIR' in environ:
+        USERDIR = environ['QUODLIBET_USERDIR']
+
+    # XXX: Exec conf.py in this directory, used to override const globals
+    # e.g. for setting USERDIR for the Windows portable version
+    # Note: execfile doesn't handle unicode paths on windows, so encode.
+    # (this doesn't use the old win api in case of str compared to os.*)
+    _CONF_PATH = os.path.join(
+        os.path.dirname(os.path.realpath(__file__)), "conf.py")
+
+    if PY2:
+        locals_ = {}
+        # FIXME: PY3PORT
+        try:
+            execfile(_CONF_PATH, globals(), locals_)
+        except IOError:
+            pass
+        else:
+            USERDIR = locals_["USERDIR"]
+
+    # XXX: users shouldn't assume the dir is there, but we currently do in
+    # some places
+    mkdir(USERDIR, 0o750)
+
+    return USERDIR
+
+
 def _fix_gst_leaks():
     """gst_element_add_pad and gst_bin_add are wrongly annotated and lead
     to PyGObject refing the passed element.
@@ -139,11 +214,57 @@ def _fix_gst_leaks():
         Gst.Element.add_pad = do_wrap(Gst.Element.add_pad)
 
 
-def _gtk_init():
+def _init_g():
+    """Call before using GdkPixbuf/GLib/Gio/GObject"""
+
+    import gi
+
+    gi.require_version("GLib", "2.0")
+    gi.require_version("Gio", "2.0")
+    gi.require_version("GObject", "2.0")
+    gi.require_version("GdkPixbuf", "2.0")
+
+    from gi.repository import GdkPixbuf
+
+    # On windows the default variants only do ANSI paths, so replace them.
+    # In some typelibs they are replaced by default, in some don't..
+    if os.name == "nt":
+        for name in ["new_from_file_at_scale", "new_from_file_at_size",
+                     "new_from_file"]:
+            cls = GdkPixbuf.Pixbuf
+            setattr(cls, name, getattr(cls, name + "_utf8", name))
+
+    # https://bugzilla.gnome.org/show_bug.cgi?id=670372
+    if not hasattr(GdkPixbuf.Pixbuf, "savev"):
+        GdkPixbuf.Pixbuf.savev = GdkPixbuf.Pixbuf.save
+
+    # Newer glib is noisy regarding deprecated signals/properties
+    # even with stable releases.
+    if is_release():
+        warnings.filterwarnings(
+            'ignore', '.* It will be removed in a future version.',
+            Warning)
+
+    # blacklist some modules, simply loading can cause segfaults
+    sys.modules["glib"] = None
+    sys.modules["gobject"] = None
+
+
+def _init_gtk():
     """Call before using Gtk/Gdk"""
 
     import gi
 
+    # pygiaio 3.14rev16 switched to fontconfig for PangoCairo. As this results
+    # in 100% CPU under win7 revert it. Maybe we need to update the
+    # cache in the windows installer for it to work... but for now revert.
+    if is_windows():
+        os.environ['PANGOCAIRO_BACKEND'] = 'win32'
+
+    # disable for consistency and trigger events seem a bit flaky here
+    if is_osx():
+        os.environ["GTK_OVERLAY_SCROLLING"] = "0"
+
     # make sure GdkX11 doesn't get used under Windows
     if os.name == "nt":
         sys.modules["gi.repository.GdkX11"] = None
@@ -156,15 +277,18 @@ def _gtk_init():
     except (ValueError, ImportError):
         pass
 
-    gi.require_version("GLib", "2.0")
     gi.require_version("Gtk", "3.0")
     gi.require_version("Gdk", "3.0")
-    gi.require_version("GObject", "2.0")
     gi.require_version("Pango", "1.0")
-    gi.require_version("GdkPixbuf", "2.0")
-    gi.require_version("Gio", "2.0")
+    gi.require_version('Soup', '2.4')
 
-    from gi.repository import Gtk, GObject, Gdk, GdkPixbuf
+    from gi.repository import Gtk, Gdk
+
+    # PyGObject doesn't fail anymore when init fails, so do it ourself
+    initialized, argv = Gtk.init_check(sys.argv)
+    if not initialized:
+        raise SystemExit("Gtk.init failed")
+    sys.argv = list(argv)
 
     # add Gtk.TreePath.__getitem__/__len__ for PyGObject 3.2
     try:
@@ -183,17 +307,11 @@ def _gtk_init():
         Gdk.EVENT_PROPAGATE = 0
         Gdk.EVENT_STOP = 1
 
-    # On windows the default variants only do ANSI paths, so replace them.
-    # In some typelibs they are replaced by default, in some don't..
-    if os.name == "nt":
-        for name in ["new_from_file_at_scale", "new_from_file_at_size",
-                     "new_from_file"]:
-            cls = GdkPixbuf.Pixbuf
-            setattr(cls, name, getattr(cls, name + "_utf8", name))
-
-    # https://bugzilla.gnome.org/show_bug.cgi?id=670372
-    if not hasattr(GdkPixbuf.Pixbuf, "savev"):
-        GdkPixbuf.Pixbuf.savev = GdkPixbuf.Pixbuf.save
+    # include our own icon theme directory
+    theme = Gtk.IconTheme.get_default()
+    theme_search_path = get_image_dir()
+    assert os.path.exists(theme_search_path)
+    theme.append_search_path(theme_search_path)
 
     # Force menu/button image related settings. We might show too many atm
     # but this makes sure we don't miss cases where we forgot to force them
@@ -210,19 +328,13 @@ def _gtk_init():
         'ignore', '.*The property GtkAlignment:[^\s]+ is deprecated.*',
         Warning)
 
-    # Newer glib is noisy regarding deprecated signals/properties
-    # even with stable releases.
-    if is_release():
-        warnings.filterwarnings(
-            'ignore', '.* It will be removed in a future version.',
-            Warning)
-
     settings = Gtk.Settings.get_default()
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         settings.set_property("gtk-button-images", True)
         settings.set_property("gtk-menu-images", True)
     if hasattr(settings.props, "gtk_primary_button_warps_slider"):
+        # https://bugzilla.gnome.org/show_bug.cgi?id=737843
         settings.set_property("gtk-primary-button-warps-slider", True)
 
     # Make sure PyGObject includes support for foreign cairo structs
@@ -277,16 +389,15 @@ def _gtk_init():
     # blacklist some modules, simply loading can cause segfaults
     sys.modules["gtk"] = None
     sys.modules["gpod"] = None
-    sys.modules["glib"] = None
-    sys.modules["gobject"] = None
     sys.modules["gnome"] = None
 
-    from quodlibet.qltk import pygobject_version
-    if pygobject_version < (3, 9):
-        GObject.threads_init()
+    from quodlibet.qltk import pygobject_version, gtk_version
+
+    MinVersions.GTK.check(gtk_version)
+    MinVersions.PYGOBJECT.check(pygobject_version)
 
 
-def _gst_init():
+def _init_gst():
     """Call once before importing GStreamer"""
 
     assert "gi.repository.Gst" not in sys.modules
@@ -329,26 +440,7 @@ def _gst_init():
         threading.Thread(target=lambda: None).start()
 
 
-def _gtk_icons_init(theme_search_path, default_icon_name=None):
-    """Register a local fallback icon theme containing our own icons.
-
-    `default_icon_name` is the icon name used for all windows if nothing
-    else is specified.
-    """
-
-    from gi.repository import Gtk
-
-    theme = Gtk.IconTheme.get_default()
-
-    assert os.path.exists(theme_search_path)
-    theme.append_search_path(theme_search_path)
-
-    if default_icon_name is not None:
-        assert theme.has_icon(default_icon_name)
-        Gtk.Window.set_default_icon_name(default_icon_name)
-
-
-def _dbus_init():
+def _init_dbus():
     """Setup dbus mainloop integration. Call before using dbus"""
 
     try:
@@ -364,18 +456,14 @@ def _dbus_init():
         DBusGMainLoop(set_as_default=True)
 
 
-def _gettext_init():
+def _init_gettext():
     """Call before using gettext helpers"""
 
-    # set by tests
-    if "QUODLIBET_NO_TRANS" in os.environ:
-        return
-
     set_i18n_envvars()
     fixup_i18n_envvars()
 
-    print_d("LANGUAGE: %r" % os.environ.get("LANGUAGE"))
-    print_d("LANG: %r" % os.environ.get("LANG"))
+    print_d("LANGUAGE: %r" % environ.get("LANGUAGE"))
+    print_d("LANG: %r" % environ.get("LANG"))
 
     try:
         locale.setlocale(locale.LC_ALL, '')
@@ -383,12 +471,13 @@ def _gettext_init():
         pass
 
     # Use the locale dir in ../build/share/locale if there is one
-    localedir = os.path.dirname(quodlibet.const.BASEDIR)
+    base_dir = get_base_dir()
+    localedir = os.path.dirname(base_dir)
     localedir = os.path.join(localedir, "build", "share", "locale")
     if not os.path.isdir(localedir) and os.name == "nt":
         # py2exe case
         localedir = os.path.join(
-            quodlibet.const.BASEDIR, "..", "..", "share", "locale")
+            base_dir, "..", "..", "share", "locale")
 
     if os.path.isdir(localedir):
         print_d("Using local localedir: %r" % unexpand(localedir))
@@ -404,65 +493,80 @@ def _gettext_init():
     else:
         print_d("Translations loaded: %r" % unexpand(t.path))
 
-    debug_text = os.environ.get("QUODLIBET_TEST_TRANS")
+    debug_text = environ.get("QUODLIBET_TEST_TRANS")
     t.install(unicode=True, debug_text=debug_text)
 
 
-def _python_init():
+def _init_python():
+    if PY2 or is_release():
+        MinVersions.PYTHON2.check(sys.version_info)
+    else:
+        # for non release builds we allow Python3
+        MinVersions.PYTHON3.check(sys.version_info)
 
-    import sys
-    if sys.version_info < MinVersions.PYTHON:
-        actual = Version(sys.version_info[:3])
-        raise ImportError("Python %s required. %s found." %
-                          (MinVersions.PYTHON, actual))
-
-    # some code depends on utf-8 default encoding (pygtk used to set it)
-    reload(sys)
-    sys.setdefaultencoding("utf-8")
-
-    __builtin__.__dict__["print_"] = print_
-    __builtin__.__dict__["print_d"] = print_d
-    __builtin__.__dict__["print_e"] = print_e
-    __builtin__.__dict__["print_w"] = print_w
-
-_python_init()
-_gettext_init()
+    builtins.__dict__["print_"] = print_
+    builtins.__dict__["print_d"] = print_d
+    builtins.__dict__["print_e"] = print_e
+    builtins.__dict__["print_w"] = print_w
 
 
-def init(icon=None, proc_title=None, name=None):
-    global quodlibet
+def _init_formats():
+    from quodlibet.formats import init
+    init()
 
-    print_d("Entering quodlibet.init")
 
-    _gtk_init()
-    _gtk_icons_init(quodlibet.const.IMAGEDIR, icon)
-    _gst_init()
-    _dbus_init()
-    _init_debug()
+def init_cli(no_translations=False):
+    """This needs to be called before any API can be used.
+    Might raise in case of an error.
 
-    from gi.repository import GLib
+    Like init() but for code not using Gtk etc.
+    """
 
-    if proc_title:
-        GLib.set_prgname(proc_title)
-        set_process_title(proc_title)
-        # Issue 736 - set after main loop has started (gtk seems to reset it)
-        GLib.idle_add(set_process_title, proc_title)
+    global _cli_initialized
 
-    if name:
-        GLib.set_application_name(name)
+    if _cli_initialized:
+        return
 
-    mkdir(quodlibet.const.USERDIR, 0750)
+    from quodlibet import config
 
-    print_d("Finished initialization.")
+    _init_python()
+    config.init_defaults()
+    if not no_translations and "QUODLIBET_NO_TRANS" not in environ:
+        _init_gettext()
+    _init_formats()
+    _init_g()
+
+    _cli_initialized = True
+
+
+def init(no_translations=False, no_excepthook=False):
+    """This needs to be called before any API can be used.
+    Might raise in case of an error.
+
+    Pass no_translations=True to disable translations (used by tests)
+    """
+
+    global _initialized
+
+    if _initialized:
+        return
+
+    init_cli(no_translations=no_translations)
+    _init_gtk()
+    _init_gtk_debug(no_excepthook=no_excepthook)
+    _init_gst()
+    _init_dbus()
+
+    _initialized = True
 
 
 def init_plugins(no_plugins=False):
     print_d("Starting plugin manager")
 
     from quodlibet import plugins
-    folders = [os.path.join(quodlibet.const.BASEDIR, "ext", kind)
+    folders = [os.path.join(get_base_dir(), "ext", kind)
                for kind in PLUGIN_DIRS]
-    folders.append(os.path.join(quodlibet.const.USERDIR, "plugins"))
+    folders.append(os.path.join(get_user_dir(), "plugins"))
     print_d("Scanning folders: %s" % folders)
     pm = plugins.init(folders, no_plugins)
     pm.rescan()
@@ -477,14 +581,6 @@ def init_plugins(no_plugins=False):
     return pm
 
 
-def init_backend(backend, librarian):
-    import quodlibet.player
-    print_d("Initializing audio backend (%s)" % backend)
-    backend = quodlibet.player.init(backend)
-    device = backend.init(librarian)
-    return device
-
-
 def enable_periodic_save(save_library):
     import quodlibet.library
     from quodlibet.util import copool
@@ -494,7 +590,7 @@ def enable_periodic_save(save_library):
 
     def periodic_config_save():
         while 1:
-            config.save(quodlibet.const.CONFIG)
+            config.save()
             yield
 
     copool.add(periodic_config_save, timeout=timeout)
@@ -540,7 +636,7 @@ def finish_first_session(app_name):
     config.set("memory", "%s_last_active_version" % app_name, const.VERSION)
 
 
-def _init_debug():
+def _init_gtk_debug(no_excepthook):
     from gi.repository import GLib
     from quodlibet.qltk.debugwindow import ExceptionDialog
 
@@ -549,7 +645,8 @@ def _init_debug():
     def _override_exceptions():
         print_d("Enabling custom exception handler.")
         sys.excepthook = ExceptionDialog.excepthook
-    GLib.idle_add(_override_exceptions)
+    if not no_excepthook:
+        GLib.idle_add(_override_exceptions)
 
     # faulthandler gives a python stacktrace on segfaults..
     try:
@@ -565,14 +662,19 @@ def _init_osx(window):
     import objc
 
     try:
+        import gi
+        gi.require_version('GtkosxApplication', '1.0')
         from gi.repository import GtkosxApplication
-        osx_app = GtkosxApplication.Application()
-    except ImportError:
+    except (ValueError, ImportError):
         print_d("importing GtkosxApplication failed, no native menus")
     else:
+        osx_app = GtkosxApplication.Application()
         window.set_as_osx_window(osx_app)
         osx_app.ready()
 
+    shared_app = NSApplication.sharedApplication()
+    gtk_delegate = shared_app.delegate()
+
     # Instead of quitting when the main window gets closed just hide it.
     # If the dock icon gets clicked we get
     # applicationShouldHandleReopen_hasVisibleWindows_ and show everything.
@@ -592,7 +694,9 @@ def _init_osx(window):
             GLib.idle_add(app.quit)
             return False
 
-    shared_app = NSApplication.sharedApplication()
+        def applicationDockMenu_(self, sender):
+            return gtk_delegate.applicationDockMenu_(sender)
+
     delegate = Delegate.alloc().init()
     delegate.retain()
     shared_app.setDelegate_(delegate)
@@ -603,10 +707,33 @@ def _init_osx(window):
             "delete-event", lambda window, event: window.hide() or True)
 
 
+def set_application_info(icon_name, process_title, app_name):
+    """Call after init() and before creating any windows to apply default
+    values for names and icons.
+    """
+
+    assert _initialized
+
+    from gi.repository import Gtk, GLib
+
+    set_process_title(process_title)
+    # Issue 736 - set after main loop has started (gtk seems to reset it)
+    GLib.idle_add(set_process_title, process_title)
+
+    GLib.set_prgname(process_title)
+    GLib.set_application_name(app_name)
+
+    theme = Gtk.IconTheme.get_default()
+    assert theme.has_icon(icon_name)
+    Gtk.Window.set_default_icon_name(icon_name)
+
+
 def main(window, before_quit=None):
     print_d("Entering quodlibet.main")
     from gi.repository import Gtk, Gdk
 
+    assert _initialized
+
     def quit_gtk(window):
 
         if before_quit is not None:
@@ -649,5 +776,13 @@ def main(window, before_quit=None):
         # if we don't show a window, startup isn't completed, so call manually
         Gdk.notify_startup_complete()
 
-    Gtk.main()
+    # set QUODLIBET_START_PERF to measure startup time until the
+    # windows is first shown.
+    if "QUODLIBET_START_PERF" in os.environ:
+        window.connect("draw", Gtk.main_quit)
+        Gtk.main()
+        sys.exit()
+    else:
+        Gtk.main()
+
     print_d("Gtk.main() done.")
