@@ -8,25 +8,25 @@
 import os
 import sys
 import bz2
-import urllib2
-import urllib
 import itertools
 
 from gi.repository import Gtk, GLib, Pango
 
-from quodlibet import const
+import quodlibet
 from quodlibet import qltk
 from quodlibet import util
 from quodlibet import config
 
-from quodlibet.browsers._base import Browser
+from quodlibet.browsers import Browser
 from quodlibet.formats.remote import RemoteFile
 from quodlibet.formats._audio import TAG_TO_SORT, MIGRATE, AudioFile
 from quodlibet.library import SongLibrary
 from quodlibet.query import Query
+from quodlibet.compat import reduce, urlopen
 from quodlibet.qltk.getstring import GetStringDialog
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.notif import Task
+from quodlibet.qltk import Icons
 from quodlibet.util import copool, connect_destroy, sanitize_tags, connect_obj
 from quodlibet.util.string import decode, encode
 from quodlibet.util.uri import URI
@@ -39,8 +39,8 @@ from quodlibet.qltk.menubutton import MenuButton
 
 STATION_LIST_URL = \
     "http://bitbucket.org/lazka/quodlibet/downloads/radiolist.bz2"
-STATIONS_FAV = os.path.join(const.USERDIR, "stations")
-STATIONS_ALL = os.path.join(const.USERDIR, "stations_all")
+STATIONS_FAV = os.path.join(quodlibet.get_user_dir(), "stations")
+STATIONS_ALL = os.path.join(quodlibet.get_user_dir(), "stations_all")
 
 # TODO: - Do the update in a thread
 #       - Ranking: reduce duplicate stations (max 3 URLs per station)
@@ -204,12 +204,13 @@ def add_station(uri):
 
     if uri.lower().endswith(".pls") or uri.lower().endswith(".m3u"):
         try:
-            sock = urllib.urlopen(uri)
-        except EnvironmentError, e:
+            sock = urlopen(uri)
+        except EnvironmentError as e:
+            encoding = util.get_locale_encoding()
             try:
-                err = e.strerror.decode(const.ENCODING, 'replace')
+                err = e.strerror.decode(encoding, 'replace')
             except (TypeError, AttributeError):
-                err = e.strerror[1].decode(const.ENCODING, 'replace')
+                err = e.strerror[1].decode(encoding, 'replace')
             qltk.ErrorMessage(None, _("Unable to add station"), err).run()
             return []
 
@@ -222,7 +223,7 @@ def add_station(uri):
     else:
         try:
             irfs = [IRFile(uri)]
-        except ValueError, err:
+        except ValueError as err:
             qltk.ErrorMessage(None, _("Unable to add station"), err).run()
 
     return irfs
@@ -239,8 +240,8 @@ def download_taglist(callback, cofuncid, step=1024 * 10):
             task.copool(cofuncid)
 
         try:
-            response = urllib2.urlopen(STATION_LIST_URL)
-        except urllib2.URLError:
+            response = urlopen(STATION_LIST_URL)
+        except EnvironmentError:
             GLib.idle_add(callback, None)
             return
 
@@ -332,7 +333,7 @@ class AddNewStation(GetStringDialog):
         super(AddNewStation, self).__init__(
             parent, _("New Station"),
             _("Enter the location of an Internet radio station:"),
-            okbutton=Gtk.STOCK_ADD)
+            button_label=_("_Add"), button_icon=Icons.LIST_ADD)
 
     def _verify_clipboard(self, text):
         # try to extract a URI from the clipboard
@@ -469,8 +470,7 @@ class QuestionBar(Gtk.InfoBar):
             bar.hide()
 
 
-class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
-    __gsignals__ = Browser.__gsignals__
+class InternetRadio(Browser, util.InstanceTracker):
 
     __stations = None
     __fav_stations = None
@@ -480,12 +480,13 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
 
     name = _("Internet Radio")
     accelerated_name = _("_Internet Radio")
+    keys = ["InternetRadio"]
     priority = 16
     uses_main_library = False
     headers = "title artist ~people grouping genre website ~format " \
         "channel-mode".split()
 
-    TYPE, STOCK, KEY, NAME = range(4)
+    TYPE, ICON_NAME, KEY, NAME = range(4)
     TYPE_FILTER, TYPE_ALL, TYPE_FAV, TYPE_SEP, TYPE_NOCAT = range(5)
     STAR = ["artist", "title", "website", "genre", "comment"]
 
@@ -529,6 +530,7 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
 
     def __init__(self, library):
         super(InternetRadio, self).__init__(spacing=12)
+        self.set_orientation(Gtk.Orientation.VERTICAL)
 
         if not self.instances():
             self._init(library)
@@ -543,16 +545,16 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
         search.connect('query-changed', self.__filter_changed)
 
         menu = Gtk.Menu()
-        new_item = MenuItem(_(u"_New Station…"), Gtk.STOCK_ADD)
+        new_item = MenuItem(_(u"_New Station…"), Icons.LIST_ADD)
         new_item.connect('activate', self.__add)
         menu.append(new_item)
-        update_item = MenuItem(_("_Update Stations"), Gtk.STOCK_REFRESH)
+        update_item = MenuItem(_("_Update Stations"), Icons.VIEW_REFRESH)
         update_item.connect('activate', self.__update)
         menu.append(update_item)
         menu.show_all()
 
         button = MenuButton(
-            SymbolicIconImage("emblem-system", Gtk.IconSize.MENU),
+            SymbolicIconImage(Icons.EMBLEM_SYSTEM, Gtk.IconSize.MENU),
             arrow=True)
         button.set_menu(menu)
 
@@ -572,19 +574,19 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
         scrolled_window.add(view)
         model = Gtk.ListStore(int, str, str, str)
 
-        model.append(row=[self.TYPE_ALL, Gtk.STOCK_DIRECTORY, "__all",
+        model.append(row=[self.TYPE_ALL, Icons.FOLDER, "__all",
                           _("All Stations")])
-        model.append(row=[self.TYPE_SEP, Gtk.STOCK_DIRECTORY, "", ""])
+        model.append(row=[self.TYPE_SEP, Icons.FOLDER, "", ""])
         #Translators: Favorite radio stations
-        model.append(row=[self.TYPE_FAV, Gtk.STOCK_DIRECTORY, "__fav",
+        model.append(row=[self.TYPE_FAV, Icons.FOLDER, "__fav",
                           _("Favorites")])
-        model.append(row=[self.TYPE_SEP, Gtk.STOCK_DIRECTORY, "", ""])
+        model.append(row=[self.TYPE_SEP, Icons.FOLDER, "", ""])
 
         filters = self.filters
         for text, k in sorted([(filters.text(k), k) for k in filters.keys()]):
-            model.append(row=[self.TYPE_FILTER, Gtk.STOCK_FIND, k, text])
+            model.append(row=[self.TYPE_FILTER, Icons.EDIT_FIND, k, text])
 
-        model.append(row=[self.TYPE_NOCAT, Gtk.STOCK_DIRECTORY,
+        model.append(row=[self.TYPE_NOCAT, Icons.FOLDER,
                           "nocat", _("No Category")])
 
         def separator(model, iter, data):
@@ -602,7 +604,7 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
         renderpb = Gtk.CellRendererPixbuf()
         renderpb.props.xpad = 3
         column.pack_start(renderpb, False)
-        column.add_attribute(renderpb, "stock_id", self.STOCK)
+        column.add_attribute(renderpb, "icon-name", self.ICON_NAME)
 
         render = Gtk.CellRendererText()
         render.set_property('ellipsize', Pango.EllipsizeMode.END)
@@ -831,11 +833,11 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
                 break
 
         iradio_items = []
-        button = MenuItem(_("Add to Favorites"), Gtk.STOCK_ADD)
+        button = MenuItem(_("Add to Favorites"), Icons.LIST_ADD)
         button.set_sensitive(in_all)
         connect_obj(button, 'activate', self.__add_fav, songs)
         iradio_items.append(button)
-        button = MenuItem(_("Remove from Favorites"), Gtk.STOCK_REMOVE)
+        button = MenuItem(_("Remove from Favorites"), Icons.LIST_REMOVE)
         button.set_sensitive(in_fav)
         connect_obj(button, 'activate', self.__remove_fav, songs)
         iradio_items.append(button)
@@ -885,6 +887,9 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
             self.__filter_changed(self.__searchbar, text)
             self.activate()
 
+    def get_filter_text(self):
+        return self.__searchbar.get_text()
+
     def activate(self):
         filter_ = self.__get_filter()
         libs = self.__get_selected_libraries()
@@ -931,8 +936,9 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
         self.view.set_cursor(path)
         self.view.scroll_to_cell(path, use_align=True, row_align=0.5)
 
-    def statusbar(self, i):
-        return ngettext("%(count)d station", "%(count)d stations", i)
+    def status_text(self, count, time=None):
+        return (ngettext("%(count)d station", "%(count)d stations", count)
+                % {'count': count})
 
 
 from quodlibet import app
