@@ -10,7 +10,7 @@ import operator
 from gi.repository import Gtk, Pango, Gdk
 
 from quodlibet import qltk
-from quodlibet.qltk.views import AllTreeView, TreeViewColumn
+from quodlibet.qltk.views import AllTreeView, TreeViewColumnButton
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk import is_accel
 from quodlibet.util import connect_obj
@@ -35,7 +35,18 @@ class Pane(AllTreeView):
 
         self.__no_fill = 0
 
-        column = TreeViewColumn(self.config.title)
+        column = TreeViewColumnButton(title=self.config.title)
+
+        def on_column_header_clicked(column, event):
+            # In case the column header gets clicked select the "All" entry
+            if event.button != Gdk.BUTTON_PRIMARY or \
+                    event.type != Gdk.EventType.BUTTON_PRESS:
+                return Gdk.EVENT_PROPAGATE
+            self.set_selected([])
+            return Gdk.EVENT_STOP
+
+        column.set_clickable(True)
+        column.connect("button-press-event", on_column_header_clicked)
         column.set_use_markup(True)
         column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
         column.set_fixed_width(50)
@@ -78,7 +89,8 @@ class Pane(AllTreeView):
 
         selection = self.get_selection()
         selection.set_mode(Gtk.SelectionMode.MULTIPLE)
-        self.__sig = selection.connect('changed', self.__selection_changed)
+        self.__sig = self.connect(
+            'selection-changed', self.__selection_changed)
         s = self.connect('popup-menu', self.__popup_menu, library)
         connect_obj(self, 'destroy', self.disconnect, s)
 
@@ -98,7 +110,7 @@ class Pane(AllTreeView):
 
     def __key_pressed(self, view, event):
         # if ctrl+a is pressed, intercept and select the All entry instead
-        if is_accel(event, "<ctrl>a"):
+        if is_accel(event, "<Primary>a"):
             self.set_selected([])
             return True
         return False
@@ -191,12 +203,12 @@ class Pane(AllTreeView):
         """Inhibit selection change events and song propagation"""
 
         self.__no_fill += 1
-        self.get_selection().handler_block(self.__sig)
+        self.handler_block(self.__sig)
 
     def uninhibit(self):
         """Uninhibit selection change events and song propagation"""
 
-        self.get_selection().handler_unblock(self.__sig)
+        self.handler_unblock(self.__sig)
         self.__no_fill -= 1
 
     def fill(self, songs):
