@@ -7,32 +7,37 @@
 # published by the Free Software Foundation
 
 import os
-import ConfigParser
+try:
+    import ConfigParser
+except ImportError:
+    import configparser as ConfigParser
 from os.path import dirname, basename
 from quodlibet.util.dprint import print_d, print_w
 
 from gi.repository import GObject
 from quodlibet.util.path import xdg_get_system_data_dirs
 
-try:
-    import dbus
-except ImportError:
-    print_w(_("Could not import %s, which is needed for "
-        "device support.") % "dbus-python")
-    dbus = None
-
-from quodlibet import const
+import quodlibet
 from quodlibet import util
 from quodlibet.devices import _udev as udev
 from quodlibet.util.importhelper import load_dir_modules
 
+try:
+    import dbus
+except ImportError:
+    if not util.is_osx() and not util.is_windows():
+        print_w(_("Could not import %s, which is needed for "
+            "device support.") % "dbus-python")
+    dbus = None
+
+
 devices = []
 
 
 def init_devices():
     global devices
 
-    load_pyc = os.name == 'nt'
+    load_pyc = util.is_windows() or util.is_osx()
     modules = load_dir_modules(dirname(__file__),
                                package=__package__,
                                load_compiled=load_pyc)
@@ -43,20 +48,21 @@ def init_devices():
         except AttributeError:
             print_w("%r doesn't contain any devices." % mod.__name__)
 
-    devices.sort()
+    devices.sort(key=lambda d: repr(d))
 
-init_devices()
 
-DEVICES = os.path.join(const.USERDIR, "devices")
+if not util.is_osx() and not util.is_windows():
+    init_devices()
+
+DEVICES = os.path.join(quodlibet.get_user_dir(), "devices")
 
 config = ConfigParser.RawConfigParser()
 config.read(DEVICES)
 
 
 def write():
-    f = file(DEVICES, 'w')
-    config.write(f)
-    f.close()
+    with open(DEVICES, 'w') as f:
+        config.write(f)
 
 
 # Return a constructor for a device given by a string
