@@ -2,7 +2,7 @@
 #
 #    Duplicates songs plugin.
 #
-#    Copyright (C) 2012, 2011 Nick Boultbee
+#    Copyright (C) 2012, 2011, 2016 Nick Boultbee
 #
 #    Finds "duplicates" of songs selected by searching the library for
 #    others with the same user-configurable "key", presenting a browser-like
@@ -21,12 +21,14 @@ from gi.repository import Gtk, Pango
 from quodlibet import app
 from quodlibet import print_d, util, qltk
 from quodlibet.plugins import PluginConfigMixin
+from quodlibet.plugins.songshelpers import any_song, is_finite
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.qltk.ccb import ConfigCheckButton
 from quodlibet.qltk.edittags import AudioFileGroup
 from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.views import RCMHintedTreeView
+from quodlibet.qltk import Icons, Button
 from quodlibet.util import connect_obj, connect_destroy
 
 
@@ -333,9 +335,7 @@ class DuplicateDialog(Gtk.Window):
             model = view.get_model()
             for row in model:
                 if view.row_expanded(row.path):
-                    for row in model:
-                        view.collapse_row(row.path)
-                    break
+                    view.collapse_row(row.path)
             else:
                 for row in model:
                     view.expand_row(row.path, False)
@@ -347,7 +347,7 @@ class DuplicateDialog(Gtk.Window):
         label = Gtk.Label(label=_("Duplicate key expression is '%s'") %
                 Duplicates.get_key_expression())
         hbox.pack_start(label, True, True, 0)
-        close = Gtk.Button(stock=Gtk.STOCK_CLOSE)
+        close = Button(_("_Close"), Icons.WINDOW_CLOSE)
         close.connect('clicked', self.__quit)
         hbox.pack_start(close, False, True, 0)
 
@@ -362,7 +362,7 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
     PLUGIN_ID = 'Duplicates'
     PLUGIN_NAME = _('Duplicates Browser')
     PLUGIN_DESC = _('Finds and displays similarly tagged versions of songs.')
-    PLUGIN_ICON = Gtk.STOCK_MEDIA_PLAY
+    PLUGIN_ICON = Icons.EDIT_SELECT_ALL
 
     MIN_GROUP_SIZE = 2
     _CFG_KEY_KEY = "key_expression"
@@ -373,6 +373,8 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
     _CFG_REMOVE_PUNCTUATION = 'remove_punctuation'
     _CFG_CASE_INSENSITIVE = 'case_insensitive'
 
+    plugin_handles = any_song(is_finite)
+
     # Cached values
     key_expression = None
     __cfg_cache = {}
@@ -460,7 +462,7 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
             if key and key in groups:
                 groups[key].add(song._song)
             elif key:
-                groups[key] = set([song._song])
+                groups[key] = {song._song}
 
         for song in app.library:
             key = self.get_key(song)
