@@ -1,8 +1,7 @@
-#! /usr/bin/env python
-#
+# -*- coding: utf-8 -*-
 #    ReplayGain Album Analysis using gstreamer rganalysis element
 #    Copyright (C) 2005,2007,2009  Michael Urman
-#                         2012,14  Nick Boultbee
+#                  2012,2014,2016  Nick Boultbee
 #                            2013  Christoph Reiter
 #
 #    This program is free software; you can redistribute it and/or modify
@@ -21,7 +20,10 @@ from quodlibet.plugins import PluginConfigMixin
 from quodlibet.browsers.collection.models import EMPTY
 
 from quodlibet.qltk.views import HintedTreeView
+from quodlibet.qltk.x import Frame
+from quodlibet.qltk import Icons, Dialog
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
+from quodlibet.plugins.songshelpers import is_writable, is_finite, each_song
 from quodlibet.util import cached_property
 
 __all__ = ['ReplayGain']
@@ -344,14 +346,15 @@ class ReplayGainPipeline(GObject.Object):
             self._next_song()
 
 
-class RGDialog(Gtk.Dialog):
+class RGDialog(Dialog):
 
     def __init__(self, albums, parent, process_mode):
         super(RGDialog, self).__init__(
-            title=_('ReplayGain Analyzer'), parent=parent,
-            buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                     Gtk.STOCK_SAVE, Gtk.ResponseType.OK)
-        )
+            title=_('ReplayGain Analyzer'), parent=parent)
+
+        self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
+        self.add_icon_button(_("_Save"), Icons.DOCUMENT_SAVE,
+                             Gtk.ResponseType.OK)
 
         self.process_mode = process_mode
         self.set_default_size(600, 400)
@@ -373,9 +376,9 @@ class RGDialog(Gtk.Dialog):
         def icon_cdf(column, cell, model, iter_, *args):
             item = model[iter_][0]
             if item.error:
-                cell.set_property('stock-id', Gtk.STOCK_DIALOG_ERROR)
+                cell.set_property('icon-name', Icons.DIALOG_ERROR)
             else:
-                cell.set_property('stock-id', None)
+                cell.set_property('icon-name', Icons.NONE)
 
         column = Gtk.TreeViewColumn()
         column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
@@ -447,10 +450,14 @@ class RGDialog(Gtk.Dialog):
 
         self.__fill_view(view, albums)
         num_to_process = sum(int(rga.should_process) for rga in self._todo)
-        template = ngettext("There is <b>%d</b> album to update (of %d)",
-                            "There are <b>%d</b> albums to update (of %d)",
-                            num_to_process)
-        info.set_markup(template % (num_to_process, len(self._todo)))
+        template = ngettext(
+            "There is <b>%(to-process)d</b> album to update (of %(all)d)",
+            "There are <b>%(to-process)d</b> albums to update (of %(all)d)",
+            num_to_process)
+        info.set_markup(template % {
+            "to-process": num_to_process,
+            "all": len(self._todo),
+        })
         self.connect("destroy", self.__destroy)
         self.connect('response', self.__response)
 
@@ -562,11 +569,11 @@ class ReplayGain(SongsMenuPlugin, PluginConfigMixin):
     PLUGIN_NAME = _('Replay Gain')
     PLUGIN_DESC = _('Analyzes and updates ReplayGain information, '
                     'using GStreamer. Results are grouped by album.')
-    PLUGIN_ICON = Gtk.STOCK_MEDIA_PLAY
-    # A little arbitrary, but seemed to match the history best.
-    PLUGIN_VERSION = '2.1'
+    PLUGIN_ICON = Icons.MULTIMEDIA_VOLUME_CONTROL
     CONFIG_SECTION = 'replaygain'
 
+    plugin_handles = each_song(is_finite, is_writable)
+
     def plugin_albums(self, albums):
         mode = self.config_get("process_if", UpdateMode.ALWAYS)
         win = RGDialog(albums, parent=self.plugin_window, process_mode=mode)
@@ -583,14 +590,6 @@ class ReplayGain(SongsMenuPlugin, PluginConfigMixin):
     def PluginPreferences(cls, parent):
         vb = Gtk.VBox(spacing=12)
 
-        # Server settings Frame
-        frame = Gtk.Frame(label=_("<b>Existing Tags</b>"))
-        frame.set_shadow_type(Gtk.ShadowType.NONE)
-        frame.get_label_widget().set_use_markup(True)
-        frame_align = Gtk.Alignment.new(0, 0, 1, 1)
-        frame_align.set_padding(6, 6, 12, 12)
-        frame.add(frame_align)
-
         # Tabulate all settings for neatness
         table = Gtk.Table(n_rows=1, n_columns=2)
         table.set_col_spacings(6)
@@ -605,7 +604,7 @@ class ReplayGain(SongsMenuPlugin, PluginConfigMixin):
 
         def create_model():
             model = Gtk.ListStore(str, str)
-            model.append([_("<b>always</b>"), UpdateMode.ALWAYS])
+            model.append(["<b>%s</b>" % _("always"), UpdateMode.ALWAYS])
             model.append([_("if <b>any</b> RG tags are missing"),
                           UpdateMode.ANY_MISSING])
             model.append([_("if <b>album</b> RG tags are missing"),
@@ -636,7 +635,9 @@ class ReplayGain(SongsMenuPlugin, PluginConfigMixin):
                          xoptions=Gtk.AttachOptions.FILL)
             table.attach(entry, 1, 2, row, row + 1)
 
-        frame_align.add(table)
+        # Server settings Frame
+        frame = Frame(_("Existing Tags"), table)
+
         vb.pack_start(frame, True, True, 0)
         return vb
 
