@@ -8,10 +8,12 @@
 
 from mutagen.mp4 import MP4, MP4Cover
 
-from quodlibet.formats._audio import AudioFile
-from quodlibet.formats._image import EmbeddedImage
 from quodlibet.util.path import get_temp_cover_file
 from quodlibet.util.string import decode
+from quodlibet.compat import iteritems
+
+from ._audio import AudioFile
+from ._image import EmbeddedImage
 
 
 class MP4File(AudioFile):
@@ -29,13 +31,19 @@ class MP4File(AudioFile):
         "\xa9grp": "grouping",
         "\xa9gen": "genre",
         "tmpo": "bpm",
-        "\xa9too": "encodedby",
+        "\xa9too": "encodedby",  # FIXME: \xa9enc should be encodedby
         "cprt": "copyright",
         "soal": "albumsort",
         "soaa": "albumartistsort",
         "soar": "artistsort",
         "sonm": "titlesort",
         "soco": "composersort",
+
+        "----:com.apple.iTunes:CONDUCTOR": "conductor",
+        "----:com.apple.iTunes:DISCSUBTITLE": "discsubtitle",
+        "----:com.apple.iTunes:LANGUAGE": "language",
+        "----:com.apple.iTunes:MOOD": "mood",
+
         "----:com.apple.iTunes:MusicBrainz Artist Id":
             "musicbrainz_artistid",
         "----:com.apple.iTunes:MusicBrainz Track Id": "musicbrainz_trackid",
@@ -50,20 +58,20 @@ class MP4File(AudioFile):
         "----:com.apple.iTunes:MusicBrainz Album Release Country":
             "releasecountry",
     }
-    __rtranslate = dict([(v, k) for k, v in __translate.iteritems()])
+    __rtranslate = dict([(v, k) for k, v in iteritems(__translate)])
 
     __tupletranslate = {
         "disk": "discnumber",
         "trkn": "tracknumber",
         }
-    __rtupletranslate = dict([(v, k) for k, v in __tupletranslate.iteritems()])
+    __rtupletranslate = dict([(v, k) for k, v in iteritems(__tupletranslate)])
 
     def __init__(self, filename):
         audio = MP4(filename)
-        self["~format"] = "%s %s" % (
-            self.format, getattr(audio.info, "codec_description", "AAC"))
-        self["~#length"] = int(audio.info.length)
+        self["~codec"] = getattr(audio.info, "codec_description", u"AAC")
+        self["~#length"] = audio.info.length
         self["~#bitrate"] = int(audio.info.bitrate / 1000)
+
         for key, values in audio.items():
             if key in self.__tupletranslate:
                 name = self.__tupletranslate[key]
@@ -208,4 +216,4 @@ class MP4File(AudioFile):
 
 info = MP4File
 types = [MP4File]
-extensions = ['.mp4', '.m4a', '.m4v']
+extensions = ['.mp4', '.m4a', '.m4v', '.3gp', '.3g2', '.3gp2']
