@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig, Michael Urman
 #           2012 Christoph Reiter
 #
@@ -10,22 +11,32 @@ import sys
 import signal
 
 import gi
+gi.require_version("Gtk", "3.0")
+
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GLib
 
+from quodlibet.util import gdecode
 
-def redraw_all_toplevels(skip=None):
-    """A hack to trigger redraws for all windows and widgets.
-    Currently uses the sensitive state which leads to the active widget
-    losing focus, so pass it with skip to skip processing it.
+
+def get_primary_accel_mod():
+    """Returns the primary Gdk.ModifierType modifier.
+
+    cmd on osx, ctrl everywhere else.
     """
 
-    if skip:
-        skip = get_top_parent(skip)
+    return Gtk.accelerator_parse("<Primary>")[1]
+
+
+def redraw_all_toplevels():
+    """A hack to trigger redraws for all windows and widgets."""
 
     for widget in Gtk.Window.list_toplevels():
-        if not widget.get_realized() or widget == skip:
+        if not widget.get_realized():
+            continue
+        if widget.is_active():
+            widget.queue_draw()
             continue
         sensitive = widget.get_sensitive()
         widget.set_sensitive(not sensitive)
@@ -73,6 +84,19 @@ def get_top_parent(widget):
         return None
 
 
+def get_menu_item_top_parent(widget):
+    """Returns the toplevel for a menu item or None if the menu
+    and none of its parents isn't attached to a widget
+    """
+
+    while isinstance(widget, Gtk.MenuItem):
+        menu = widget.get_parent()
+        if not menu:
+            return
+        widget = menu.get_attach_widget()
+    return get_top_parent(widget)
+
+
 def find_widgets(container, type_):
     """Given a container, find all children that are a subclass of type_
     (including itself)
@@ -140,15 +164,38 @@ def _popup_menu_at_widget(menu, widget, button, time, under):
 
         return (menu_x, menu_y, True) # x, y, move_within_screen
     menu_popup(menu, None, None, pos_func, None, button, time)
-    return True
+
+
+def _ensure_menu_attached(menu, widget):
+    assert widget is not None
+
+    # Workaround the menu inheriting the wrong colors with the Ubuntu 12.04
+    # default themes. Attaching to the parent kinda works... submenus still
+    # have the wrong color.
+    if isinstance(widget, Gtk.Button):
+        widget = widget.get_parent() or widget
+
+    attached_widget = menu.get_attach_widget()
+    if attached_widget is widget:
+        return
+    if attached_widget is not None:
+        menu.detach()
+    menu.attach_to_widget(widget, None)
 
 
 def popup_menu_under_widget(menu, widget, button, time):
-    return _popup_menu_at_widget(menu, widget, button, time, True)
+    _ensure_menu_attached(menu, widget)
+    _popup_menu_at_widget(menu, widget, button, time, True)
 
 
 def popup_menu_above_widget(menu, widget, button, time):
-    return _popup_menu_at_widget(menu, widget, button, time, False)
+    _ensure_menu_attached(menu, widget)
+    _popup_menu_at_widget(menu, widget, button, time, False)
+
+
+def popup_menu_at_widget(menu, widget, button, time):
+    _ensure_menu_attached(menu, widget)
+    menu_popup(menu, None, None, None, None, button, time)
 
 
 def add_fake_accel(widget, accel):
@@ -227,6 +274,20 @@ def is_wayland():
     return False
 
 
+def get_backend_name():
+    """The GDK backend name"""
+
+    display = Gdk.Display.get_default()
+    if display is not None:
+        name = gdecode(display.__gtype__.name)
+        if name.startswith("Gdk"):
+            name = name[3:]
+        if name.endswith("Display"):
+            name = name[:-7]
+        return name
+    return u"Unknown"
+
+
 gtk_version = (Gtk.get_major_version(), Gtk.get_minor_version(),
                Gtk.get_micro_version())
 
@@ -332,7 +393,9 @@ def add_signal_watch(signal_action):
 # Legacy plugin/code support.
 from quodlibet.qltk.msg import *
 from quodlibet.qltk.x import *
+from quodlibet.qltk.icons import Icons
 from quodlibet.qltk.window import Window, UniqueWindow
 
 Window
 UniqueWindow
+Icons
