@@ -15,6 +15,8 @@ from quodlibet import util
 
 from quodlibet.util import connect_obj
 from quodlibet.qltk.views import AllTreeView
+from quodlibet.qltk.x import MenuItem
+from quodlibet.qltk import Icons
 
 
 class DownloadWindow(qltk.UniqueWindow):
@@ -89,7 +91,7 @@ class DownloadWindow(qltk.UniqueWindow):
         if model:
             iters = map(model.get_iter, paths)
             menu = Gtk.Menu()
-            item = Gtk.ImageMenuItem(Gtk.STOCK_STOP)
+            item = MenuItem(_("_Stop"), Icons.PROCESS_STOP)
             connect_obj(item, 'activate', self.__stop_download, iters)
             menu.append(item)
             menu.connect('selection-done', lambda m: m.destroy())
@@ -139,7 +141,7 @@ class DownloadWindow(qltk.UniqueWindow):
         return True
 
     def _download(self, source, target):
-        fileobj = file(target, "wb")
+        fileobj = open(target, "wb")
         self.downloads.append(row=[None, fileobj, 0, source])
         self.__start_next()
 
