@@ -14,6 +14,7 @@ from gi.repository import Gtk, Gdk, Pango
 from quodlibet import qltk
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.x import SeparatorMenuItem, Align
+from quodlibet.qltk import Icons
 from quodlibet.util import connect_destroy
 
 from quodlibet.pattern import XMLFromMarkupPattern
@@ -50,6 +51,7 @@ class SongInfo(Gtk.EventBox):
         align = Align(halign=Gtk.Align.START, valign=Gtk.Align.START)
         label = Gtk.Label()
         label.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
+        label.set_track_visited_links(False)
         label.set_selectable(True)
         align.add(label)
         label.set_alignment(0.0, 0.0)
@@ -101,18 +103,19 @@ class SongInfo(Gtk.EventBox):
             menu.append(sub)
 
     def _get_menu(self, player, library):
-        item = qltk.MenuItem(_(u"_Edit Display…"), Gtk.STOCK_EDIT)
+        item = qltk.MenuItem(_(u"_Edit Display…"), Icons.EDIT)
         item.connect('activate', self._on_edit_display, player)
 
         songs = [player.song] if player.song else []
-        song_menu = SongsMenu(library, songs, remove=False,
+        song_menu = SongsMenu(library, songs, remove=False, delete=True,
                               accels=False, items=[[item]])
 
         song_menu.show_all()
         return song_menu
 
     def _on_edit_display(self, menu_item, player):
-        editor = PatternEdit(self, SongInfo._pattern)
+        editor = PatternEdit(
+            self, SongInfo._pattern, alternative_markup=True, links=True)
         editor.text = self._pattern
         editor.apply.connect('clicked', self._on_set_pattern, editor, player)
         editor.show()
