@@ -9,14 +9,15 @@ import os
 
 from gi.repository import Gtk, GObject, GLib
 
+import quodlibet
 from quodlibet import config
-from quodlibet import const
 
-from quodlibet.parse import Query
+from quodlibet.query import Query
 from quodlibet.qltk.cbes import ComboBoxEntrySave
+from quodlibet.qltk.entry import QueryValidator
 from quodlibet.qltk.ccb import ConfigCheckMenuItem
 from quodlibet.qltk.x import SeparatorMenuItem
-from quodlibet.util import limit_songs, DeferredSignal
+from quodlibet.util import limit_songs, DeferredSignal, gdecode
 
 
 class SearchBarBox(Gtk.HBox):
@@ -41,11 +42,12 @@ class SearchBarBox(Gtk.HBox):
         super(SearchBarBox, self).__init__(spacing=6)
 
         if filename is None:
-            filename = os.path.join(const.USERDIR, "lists", "queries")
+            filename = os.path.join(
+                quodlibet.get_user_dir(), "lists", "queries")
 
         combo = ComboBoxEntrySave(filename, count=8,
-                validator=Query.is_valid_color, title=_("Saved Searches"),
-                edit_title=_("Edit saved searches..."))
+                validator=QueryValidator, title=_("Saved Searches"),
+                edit_title=_(u"Edit saved searches…"))
 
         self.__deferred_changed = DeferredSignal(
             self.__filter_changed, timeout=self.timeout, owner=self)
@@ -73,7 +75,7 @@ class SearchBarBox(Gtk.HBox):
         self.pack_start(combo, True, True, 0)
 
         if accel_group:
-            key, mod = Gtk.accelerator_parse("<ctrl>L")
+            key, mod = Gtk.accelerator_parse("<Primary>L")
             accel_group.connect(key, mod, 0,
                     lambda *x: entry.mnemonic_activate(True))
 
@@ -93,7 +95,7 @@ class SearchBarBox(Gtk.HBox):
     def get_text(self):
         """Get the active text as unicode"""
 
-        return self.__entry.get_text().decode("utf-8")
+        return gdecode(self.__entry.get_text())
 
     def changed(self):
         """Triggers a filter-changed signal if the current text
