@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig
 #           2012 Christoph Reiter
-#      2011-2013 Nick Boultbee
 #           2014 Jan Path
+#      2011-2016 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -23,6 +23,7 @@ from quodlibet.qltk.properties import SongProperties
 from quodlibet.qltk.views import AllTreeView, DragScroll
 from quodlibet.qltk.ratingsmenu import ConfirmRateMultipleDialog
 from quodlibet.qltk.songmodel import PlaylistModel
+from quodlibet.qltk import Icons
 from quodlibet.util.uri import URI
 from quodlibet.formats._audio import TAG_TO_SORT, AudioFile
 from quodlibet.qltk.x import SeparatorMenuItem
@@ -35,9 +36,10 @@ DND_QL, DND_URI_LIST = range(2)
 
 class SongInfoSelection(GObject.Object):
     """
-    InfoSelection: Songs which get included in the status bar
-    summary. changed gets fired after any of the songs in the
-    selection or the selection it self have changed.
+    Songs which get included in the status bar summary.
+
+    The `changed` signal gets fired after any of the songs in the
+    selection or the selection it self has changed.
     The signal is async.
 
     Two selection states:
@@ -46,14 +48,13 @@ class SongInfoSelection(GObject.Object):
 
     The signals fires if the state changes.
 
-    FIXME:
-        row-changed for song lists isn't implemented (performance).
-        Since a library change could change the selection it should
-        also trigger a this.
+    FIXME: `row-changed` for song lists isn't implemented (performance).
+            Since a library change could change the selection it should
+            also trigger this.
 
-        Since this would happen quite often (song stat changes) and
-        would lead to a complete recalc in the common case ignore it for
-        now.
+            Since this would happen quite often (song stat changes) and
+            would lead to a complete re-calc in the common case,
+            ignore it for now.
     """
 
     __gsignals__ = {
@@ -68,10 +69,11 @@ class SongInfoSelection(GObject.Object):
         self.__songlist = songlist
         self.__selection = sel = songlist.get_selection()
         self.__count = sel.count_selected_rows()
-        self.__sel_id = sel.connect('changed', self.__selection_changed_cb)
+        self.__sel_id = songlist.connect(
+            'selection-changed', self.__selection_changed_cb)
 
     def destroy(self):
-        self.__selection.disconnect(self.__sel_id)
+        self.__songlist.disconnect(self.__sel_id)
         if self.__idle:
             GLib.source_remove(self.__idle)
 
@@ -89,7 +91,6 @@ class SongInfoSelection(GObject.Object):
                 songs = self.__songlist.get_selected_songs()
         self.emit('changed', songs)
         self.__idle = None
-        False
 
     def __emit_info_selection(self, songs=None):
         if self.__idle:
@@ -97,7 +98,7 @@ class SongInfoSelection(GObject.Object):
         self.__idle = GLib.idle_add(
             self.__idle_emit, songs, priority=GLib.PRIORITY_LOW)
 
-    def __selection_changed_cb(self, selection):
+    def __selection_changed_cb(self, songlist, selection):
         count = selection.count_selected_rows()
         if self.__count == count == 0:
             return
@@ -113,19 +114,22 @@ def get_columns():
     """Gets the list of songlist column headings"""
 
     if config.has_option("settings", "columns"):
-        return config.getstringlist(
+        columns = config.getstringlist(
             "settings", "columns", const.DEFAULT_COLUMNS)
     else:
         # migrate old settings
         try:
             columns = config.get("settings", "headers").split()
         except config.Error:
-            return const.DEFAULT_COLUMNS
+            columns = const.DEFAULT_COLUMNS
         else:
             config.remove_option("settings", "headers")
             set_columns(columns)
             config.setstringlist("settings", "columns", columns)
-            return columns
+
+    if "~current" in columns:
+        columns.remove("~current")
+    return columns
 
 
 def set_columns(vals):
@@ -151,9 +155,12 @@ def get_sort_tag(tag):
     elif tag == "~album~discsubtitle":
         tag = "album"
 
-    if tag.startswith("<"):
+    if "<" in tag:
         for key, value in replace_order.iteritems():
             tag = tag.replace("<%s>" % key, "<%s>" % value)
+        for key, value in TAG_TO_SORT.iteritems():
+            tag = tag.replace("<%s>" % key,
+                               "<{1}|<{1}>|<{0}>>".format(key, value))
         tag = Pattern(tag).format
     else:
         tags = util.tagsplit(tag)
@@ -349,7 +356,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             # Translators: The substituted string is the name of the
             # selected column (a translated tag name).
             b = qltk.MenuItem(
-                _("_Filter on %s") % util.tag(t, True), Gtk.STOCK_INDEX)
+                _("_Filter on %s") % util.tag(t, True), Icons.EDIT_SELECT_ALL)
             b.connect('activate', self.__filter_on, t, songs, browser)
             return b
 
@@ -617,16 +624,18 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
 
     def __key_press(self, songlist, event, librarian):
         rating_accels = [
-            "<ctrl>%d" % i for i in range(min(10, config.RATINGS.number + 1))]
+            "<Primary>%d" % i for i in range(
+                min(10, config.RATINGS.number + 1))]
 
-        if qltk.is_accel(event, *rating_accels):
+        if (qltk.is_accel(event, *rating_accels) and
+                config.getboolean("browsers", "rating_hotkeys")):
             rating = int(chr(event.keyval)) * config.RATINGS.precision
             self.__set_rating(rating, self.get_selected_songs(), librarian)
             return True
-        elif qltk.is_accel(event, "<ctrl>Return", "<ctrl>KP_Enter"):
+        elif qltk.is_accel(event, "<Primary>Return", "<Primary>KP_Enter"):
             self.__enqueue(self.get_selected_songs())
             return True
-        elif qltk.is_accel(event, "<control>F"):
+        elif qltk.is_accel(event, "<Primary>F"):
             self.emit('start-interactive-search')
             return True
         elif qltk.is_accel(event, "<alt>Return"):
@@ -635,7 +644,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
                 window = SongProperties(librarian, songs, parent=self)
                 window.show()
             return True
-        elif qltk.is_accel(event, "<control>I"):
+        elif qltk.is_accel(event, "<Primary>I"):
             songs = self.get_selected_songs()
             if songs:
                 window = Information(librarian, songs, self)
@@ -997,7 +1006,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
                 event = Gtk.get_current_event()
                 if event:
                     ok, state = event.get_state()
-                    if ok and state & Gdk.ModifierType.CONTROL_MASK:
+                    if ok and state & qltk.get_primary_accel_mod():
                         ctrl_held = True
 
                 self.toggle_column_sort(column, replace=not ctrl_held)
@@ -1026,12 +1035,13 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         current_set = set(current)
 
         def tag_title(tag):
-            if tag.startswith("<"):
+            if "<" in tag:
                 return util.pattern(tag)
             return util.tag(tag)
         current = zip(map(tag_title, current), current)
 
-        def add_header_toggle(menu, (header, tag), active, column=column):
+        def add_header_toggle(menu, pair, active, column=column):
+            header, tag = pair
             item = Gtk.CheckMenuItem(label=header)
             item.tag = tag
             item.set_active(active)
@@ -1048,19 +1058,20 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         menu.append(sep)
 
         trackinfo = """title genre ~title~version ~#track
-            ~#playcount ~#skipcount ~rating ~#length""".split()
+            ~#playcount ~#skipcount ~rating ~#length ~playlists""".split()
         peopleinfo = """artist ~people performer arranger author composer
             conductor lyricist originalartist""".split()
         albuminfo = """album ~album~discsubtitle labelid ~#disc ~#discs
             ~#tracks albumartist""".split()
-        dateinfo = """date originaldate recordingdate ~#laststarted
-            ~#lastplayed ~#added ~#mtime""".split()
+        dateinfo = """date originaldate recordingdate ~year ~originalyear
+            ~#laststarted ~#lastplayed ~#added ~#mtime""".split()
         fileinfo = """~format ~#bitrate ~#filesize ~filename ~basename ~dirname
-            ~uri""".split()
+            ~uri ~codec ~encoding""".split()
         copyinfo = """copyright organization location isrc
             contact website""".split()
-        all_headers = reduce(lambda x, y: x + y,
-            [trackinfo, peopleinfo, albuminfo, dateinfo, fileinfo, copyinfo])
+        all_headers = sum(
+            [trackinfo, peopleinfo, albuminfo, dateinfo, fileinfo, copyinfo],
+            [])
 
         for name, group in [
             (_("All _Headers"), all_headers),
@@ -1155,7 +1166,8 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             return True
 
         widget = column.get_widget()
-        return qltk.popup_menu_under_widget(menu, widget, 3, time)
+        qltk.popup_menu_under_widget(menu, widget, 3, time)
+        return True
 
 
 @config.register_upgrade_function
