@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # Copyright 2006 Joe Wreschnig
-#      2013-2014 Nick Boultbee
+#      2013-2016 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -7,6 +8,8 @@
 
 from gi.repository import Gtk
 
+from quodlibet.qltk.pluginwin import PluginWindow
+
 from quodlibet import qltk
 
 from quodlibet.util import print_exc
@@ -14,9 +17,9 @@ from quodlibet.qltk.msg import WarningMessage
 from quodlibet.qltk.delete import TrashMenuItem, trash_songs
 from quodlibet.qltk.information import Information
 from quodlibet.qltk.properties import SongProperties
-from quodlibet.qltk.x import SeparatorMenuItem, Button
-from quodlibet.qltk import get_top_parent
-from quodlibet.util import connect_obj
+from quodlibet.qltk.x import SeparatorMenuItem
+from quodlibet.qltk.ratingsmenu import RatingsMenuItem
+from quodlibet.qltk import get_top_parent, get_menu_item_top_parent, Icons
 from quodlibet.plugins import PluginManager, PluginHandler
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.util.songwrapper import ListWrapper, check_wrapper_changed
@@ -28,19 +31,18 @@ class ConfirmMultiSongInvoke(WarningMessage):
     RESPONSE_INVOKE = 1
 
     def __init__(self, parent, plugin_name, count):
-        title = ngettext("Run the plugin \"%s\" on %d song?",
-                         "Run the plugin \"%s\" on %d songs?",
-                         count) % (plugin_name, count)
+        title = ngettext("Run the plugin \"%(name)s\" on %(count)d song?",
+                         "Run the plugin \"%(name)s\" on %(count)d songs?",
+                         count) % {"name": plugin_name, "count": count}
 
         super(ConfirmMultiSongInvoke, self).__init__(
             get_top_parent(parent),
             title, "",
             buttons=Gtk.ButtonsType.NONE)
 
-        self.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-        delete_button = Button(_("_Run Plugin"), Gtk.STOCK_EXECUTE)
-        delete_button.show()
-        self.add_action_widget(delete_button, self.RESPONSE_INVOKE)
+        self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
+        self.add_icon_button(_("_Run Plugin"), Icons.SYSTEM_RUN,
+                             self.RESPONSE_INVOKE)
         self.set_default_response(Gtk.ResponseType.CANCEL)
 
     @classmethod
@@ -57,19 +59,18 @@ class ConfirmMultiAlbumInvoke(WarningMessage):
     RESPONSE_INVOKE = 1
 
     def __init__(self, parent, plugin_name, count):
-        title = ngettext("Run the plugin \"%s\" on %d album?",
-                         "Run the plugin \"%s\" on %d albums?",
-                         count) % (plugin_name, count)
+        title = ngettext("Run the plugin \"%(name)s\" on %(count)d album?",
+                         "Run the plugin \"%(name)s\" on %(count)d albums?",
+                         count) % {'name': plugin_name, 'count': count}
 
         super(ConfirmMultiAlbumInvoke, self).__init__(
             get_top_parent(parent),
             title, "",
             buttons=Gtk.ButtonsType.NONE)
 
-        self.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-        delete_button = Button(_("_Run Plugin"), Gtk.STOCK_EXECUTE)
-        delete_button.show()
-        self.add_action_widget(delete_button, self.RESPONSE_INVOKE)
+        self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
+        self.add_icon_button(_("_Run Plugin"), Icons.SYSTEM_RUN,
+                             self.RESPONSE_INVOKE)
         self.set_default_response(Gtk.ResponseType.CANCEL)
 
     @classmethod
@@ -95,9 +96,8 @@ class SongsMenuPluginHandler(PluginHandler):
         if album_confirmer is not None:
             self._confirm_multiple_albums = album_confirmer
 
-    def Menu(self, library, parent, songs):
+    def Menu(self, library, songs):
         songs = ListWrapper(songs)
-        parent = qltk.get_top_parent(parent)
 
         attrs = ['plugin_song', 'plugin_songs',
                  'plugin_album', 'plugin_albums']
@@ -120,7 +120,7 @@ class SongsMenuPluginHandler(PluginHandler):
             usable = max([callable(getattr(Kind, s)) for s in attrs])
             if usable:
                 try:
-                    items.append(Kind(songs, library, parent))
+                    items.append(Kind(songs, library))
                 except:
                     print_e("Couldn't initialise song plugin %s. Stack trace:"
                             % Kind)
@@ -132,16 +132,21 @@ class SongsMenuPluginHandler(PluginHandler):
             for item in items:
                 try:
                     menu.append(item)
-                    args = (library, parent, songs)
+                    args = (library, songs)
                     if item.get_submenu():
                         for subitem in item.get_submenu().get_children():
-                            connect_obj(subitem,
-                                'activate', self.__handle, item, *args)
+                            subitem.connect(
+                                'activate', self.__on_activate, item, *args)
                     else:
-                        item.connect('activate', self.__handle, *args)
+                        item.connect(
+                            'activate', self.__on_activate, item, *args)
                 except:
                     print_exc()
                     item.destroy()
+            menu.append(SeparatorMenuItem())
+            prefs = Gtk.MenuItem(label=_("Configure Plugins…"))
+            prefs.connect("activate", lambda _: PluginWindow().show())
+            menu.append(prefs)
 
         else:
             menu = None
@@ -163,18 +168,24 @@ class SongsMenuPluginHandler(PluginHandler):
     def handle(self, plugin_id, library, parent, songs):
         """Start a song menu plugin directly without a menu"""
 
+        parent = get_top_parent(parent)
+
         for plugin in self.__plugins:
             if plugin.PLUGIN_ID == plugin_id:
                 songs = ListWrapper(songs)
                 try:
-                    plugin = plugin(songs, library, parent)
+                    plugin = plugin(songs, library)
                 except Exception:
                     print_exc()
                 else:
-                    self.__handle(plugin, library, parent, songs)
+                    self.__handle(plugin, plugin, library, songs, parent)
                 return
 
-    def __handle(self, plugin, library, parent, songs):
+    def __on_activate(self, item, plugin, library, songs):
+        parent = get_menu_item_top_parent(item)
+        self.__handle(item, plugin, library, songs, parent)
+
+    def __handle(self, item, plugin, library, songs, parent):
         if len(songs) == 0:
             return
 
@@ -265,17 +276,31 @@ class SongsMenu(Gtk.Menu):
 
     def __init__(self, library, songs, plugins=True, playlists=True,
                  queue=True, devices=True, remove=True, delete=False,
-                 edit=True, parent=None):
+                 edit=True, ratings=True, items=None, accels=True):
         super(SongsMenu, self).__init__()
 
         # The library may actually be a librarian; if it is, use it,
         # otherwise find the real librarian.
         librarian = getattr(library, 'librarian', library)
 
+        if ratings:
+            ratings_item = RatingsMenuItem(songs, librarian)
+            ratings_item.set_sensitive(bool(songs))
+            self.append(ratings_item)
+            self.separate()
+
+        # external item groups
+        for subitems in reversed(items or []):
+            self.separate()
+            for item in subitems:
+                self.append(item)
+            self.separate()
+
         if plugins:
-            submenu = self.plugins.Menu(librarian, parent, songs)
+            submenu = self.plugins.Menu(librarian, songs)
             if submenu is not None:
-                b = qltk.MenuItem(_("_Plugins"), Gtk.STOCK_EXECUTE)
+                b = qltk.MenuItem(_("_Plugins"), Icons.SYSTEM_RUN)
+                b.set_sensitive(bool(songs))
                 self.append(b)
                 b.set_submenu(submenu)
                 self.append(SeparatorMenuItem())
@@ -291,8 +316,6 @@ class SongsMenu(Gtk.Menu):
             if not song.is_file:
                 is_file = False
 
-        self.separate()
-
         if playlists:
             # Needed here to avoid a circular import; most browsers use
             # a SongsMenu, but SongsMenu needs access to the playlist
@@ -302,20 +325,28 @@ class SongsMenu(Gtk.Menu):
             # some kind of inversion of control here.
             from quodlibet.browsers.playlists.menu import PlaylistMenu
             try:
-                submenu = PlaylistMenu(songs, parent)
+                submenu = PlaylistMenu(songs)
             except AttributeError as e:
                 print_w("Couldn't get Playlists menu: %s" % e)
             else:
-                b = qltk.MenuItem(_("Play_lists"), Gtk.STOCK_ADD)
-                b.set_sensitive(can_add)
+                b = qltk.MenuItem(_("Play_lists"), Icons.LIST_ADD)
+                b.set_sensitive(can_add and bool(songs))
                 b.set_submenu(submenu)
                 self.append(b)
         if queue:
-            b = qltk.MenuItem(_("Add to _Queue"), Gtk.STOCK_ADD)
-            b.connect('activate', self.__enqueue, songs)
-            qltk.add_fake_accel(b, "<ctrl>Return")
+            b = qltk.MenuItem(_("Add to _Queue"), Icons.LIST_ADD)
+
+            def enqueue_cb(item, songs):
+                songs = filter(lambda s: s.can_add, songs)
+                if songs:
+                    from quodlibet import app
+                    app.window.playlist.enqueue(songs)
+
+            b.connect('activate', enqueue_cb, songs)
+            if accels:
+                qltk.add_fake_accel(b, "<Primary>Return")
             self.append(b)
-            b.set_sensitive(can_add)
+            b.set_sensitive(can_add and bool(songs))
 
         if devices:
             from quodlibet import browsers
@@ -326,8 +357,9 @@ class SongsMenu(Gtk.Menu):
             else:
                 if browsers.media.MediaDevices in browsers.browsers:
                     submenu = browsers.media.Menu(songs, library)
-                    b = qltk.MenuItem(_("_Copy to Device"), Gtk.STOCK_COPY)
-                    b.set_sensitive(can_add and len(submenu) > 0)
+                    b = qltk.MenuItem(_("_Copy to Device"), Icons.EDIT_COPY)
+                    b.set_sensitive(
+                        can_add and len(submenu) > 0 and bool(songs))
                     b.set_submenu(submenu)
                     self.append(b)
 
@@ -335,41 +367,54 @@ class SongsMenu(Gtk.Menu):
             self.separate()
 
         if remove:
-            b = qltk.MenuItem(_("_Remove from library"), Gtk.STOCK_REMOVE)
+            b = qltk.MenuItem(_("_Remove from Library"), Icons.LIST_REMOVE)
             if callable(remove):
-                connect_obj(b, 'activate', remove, songs)
+                b.connect('activate', lambda item: remove(songs))
             else:
-                b.connect('activate', self.__remove, songs, library)
-                b.set_sensitive(in_lib)
+                def remove_cb(item, songs, library):
+                    library.remove(set(songs))
+
+                b.connect('activate', remove_cb, songs, library)
+                b.set_sensitive(in_lib and bool(songs))
             self.append(b)
 
         if delete:
             if callable(delete):
-                b = Gtk.ImageMenuItem(Gtk.STOCK_DELETE, use_stock=True)
-                connect_obj(b, 'activate', delete, songs)
+                b = qltk.MenuItem(_("_Delete"), Icons.EDIT_DELETE)
+                b.connect('activate', lambda item: delete(songs))
             else:
                 b = TrashMenuItem()
-                connect_obj(b, 'activate', trash_songs,
-                                 parent, songs, librarian)
-                b.set_sensitive(is_file)
+
+                def trash_cb(item):
+                    parent = get_menu_item_top_parent(item)
+                    trash_songs(parent, songs, librarian)
+
+                b.connect('activate', trash_cb)
+                b.set_sensitive(is_file and bool(songs))
             self.append(b)
 
         if edit:
             self.separate()
-            b = qltk.MenuItem(_("Edit _Tags"), Gtk.STOCK_PROPERTIES)
-            qltk.add_fake_accel(b, "<alt>Return")
+            b = qltk.MenuItem(_("Edit _Tags"), Icons.DOCUMENT_PROPERTIES)
+            b.set_sensitive(bool(songs))
+            if accels:
+                qltk.add_fake_accel(b, "<alt>Return")
 
             def song_properties_cb(menu_item):
+                parent = get_menu_item_top_parent(menu_item)
                 window = SongProperties(librarian, songs, parent)
                 window.show()
 
             b.connect('activate', song_properties_cb)
             self.append(b)
 
-            b = Gtk.ImageMenuItem(label=Gtk.STOCK_INFO, use_stock=True)
-            qltk.add_fake_accel(b, "<ctrl>I")
+            b = qltk.MenuItem(_("_Information"), Icons.DIALOG_INFORMATION)
+            b.set_sensitive(bool(songs))
+            if accels:
+                qltk.add_fake_accel(b, "<Primary>I")
 
             def information_cb(menu_item):
+                parent = get_menu_item_top_parent(menu_item)
                 window = Information(librarian, songs, parent)
                 window.show()
             b.connect('activate', information_cb)
@@ -391,12 +436,3 @@ class SongsMenu(Gtk.Menu):
             return
         elif not isinstance(self.get_children()[0], Gtk.SeparatorMenuItem):
             self.prepend(SeparatorMenuItem())
-
-    def __remove(self, item, songs, library):
-        library.remove(set(songs))
-
-    def __enqueue(self, item, songs):
-        songs = filter(lambda s: s.can_add, songs)
-        if songs:
-            from quodlibet import app
-            app.window.playlist.enqueue(songs)
