@@ -13,12 +13,10 @@ import ctypes
 import ctypes.util
 import sys
 import traceback
-import urlparse
 import unicodedata
 import threading
 import subprocess
 import webbrowser
-import contextlib
 
 # Windows doesn't have fcntl, just don't lock for now
 try:
@@ -27,11 +25,35 @@ try:
 except ImportError:
     fcntl = None
 
-from quodlibet.util.path import iscommand, is_fsnative
+from quodlibet.compat import reraise as py_reraise, urlparse, PY2, text_type, \
+    iteritems
+from quodlibet.util.path import iscommand
 from quodlibet.util.string.titlecase import title
 
 from quodlibet.const import SUPPORT_EMAIL, COPYRIGHT
 from quodlibet.util.dprint import print_d, print_
+from .misc import environ, argv, cached_func, get_locale_encoding, \
+    get_fs_encoding
+from .environment import *
+from .enum import enum
+
+
+# pyflakes
+environ, argv, cached_func, get_locale_encoding, get_fs_encoding, enum
+
+
+if PY2:
+    def gdecode(s):
+        """Returns unicode for the glib text type"""
+
+        assert isinstance(s, bytes)
+        return s.decode("utf-8")
+else:
+    def gdecode(s):
+        """Returns unicode for the glib text type"""
+
+        assert isinstance(s, text_type)
+        return s
 
 
 class InstanceTracker(object):
@@ -147,7 +169,7 @@ class OptionParser(object):
         from getopt import getopt, GetoptError
         try:
             opts, args = getopt(args, self.__shorts(), self.__longs())
-        except GetoptError, s:
+        except GetoptError as s:
             s = str(s)
             text = []
             if "not recognized" in s:
@@ -312,6 +334,10 @@ def format_rating(value, blank=True):
     return prefs.full_symbol * ons + prefs.blank_symbol * offs
 
 
+def format_bitrate(value):
+    return _("%d kbps") % int(value)
+
+
 def format_size(size):
     """Turn an integer size value into something human-readable."""
     # TODO: Better i18n of this (eg use O/KO/MO/GO in French)
@@ -402,7 +428,8 @@ def _split_numeric_sortkey(s, limit=10,
     it can be used for human sorting. Also removes all extra whitespace."""
     result = reg(s)
     if not result or not limit:
-        return (join(s.split()),)
+        text = join(s.split())
+        return (text,) if text else tuple()
     else:
         start, end = result.span()
         return (
@@ -412,10 +439,12 @@ def _split_numeric_sortkey(s, limit=10,
 
 
 def human_sort_key(s, normalize=unicodedata.normalize):
-    if not isinstance(s, unicode):
+    if not s:
+        return ()
+    if not isinstance(s, text_type):
         s = s.decode("utf-8")
     s = normalize("NFD", s.lower())
-    return s and _split_numeric_sortkey(s)
+    return _split_numeric_sortkey(s)
 
 
 def website(site):
@@ -435,7 +464,7 @@ def website(site):
 
     # sensible-browser is a debian thing
     blocking_progs = ["sensible-browser"]
-    blocking_progs.extend(os.environ.get("BROWSER", "").split(":"))
+    blocking_progs.extend(environ.get("BROWSER", "").split(":"))
 
     for prog in blocking_progs:
         if not iscommand(prog):
@@ -501,10 +530,12 @@ def tagsplit(tag):
         return [tag]
 
 
-def pattern(pat, cap=True, esc=False):
+def pattern(pat, cap=True, esc=False, markup=False):
     """Return a 'natural' version of the pattern string for human-readable
-    bits. Assumes all tags in the pattern are present."""
-    from quodlibet.pattern import Pattern, XMLFromPattern
+    bits. Assumes all tags in the pattern are present.
+    """
+
+    from quodlibet.pattern import Pattern, XMLFromPattern, XMLFromMarkupPattern
 
     class Fakesong(dict):
         cap = False
@@ -514,13 +545,20 @@ def pattern(pat, cap=True, esc=False):
 
         def list(self, key):
             return [tag(k, self.cap) for k in tagsplit(key)]
-        list_seperate = list
-        __call__ = comma
+        list_separate = list
+
+        def __call__(self, tag, *args):
+            return 0 if '~#' in tag[:2] else self.comma(tag)
 
     fakesong = Fakesong({'filename': tag('filename', cap)})
     fakesong.cap = cap
     try:
-        p = (esc and XMLFromPattern(pat)) or Pattern(pat)
+        if markup:
+            p = XMLFromMarkupPattern(pat)
+        elif esc:
+            p = XMLFromPattern(pat)
+        else:
+            p = Pattern(pat)
     except ValueError:
         return _("Invalid pattern")
 
@@ -556,7 +594,7 @@ def fver(tup):
 
 
 def uri_is_valid(uri):
-    return bool(urlparse.urlparse(uri)[0])
+    return bool(urlparse(uri)[0])
 
 
 def make_case_insensitive(filename):
@@ -565,8 +603,11 @@ def make_case_insensitive(filename):
 
 def print_exc(limit=None, file=None):
     """A wrapper preventing crashes on broken pipes in print_exc."""
-    if not file:
-        file = sys.stderr
+    if file is None:
+        if PY2:
+            file = sys.stderr
+        else:
+            file = sys.stderr.buffer
     print_(traceback.format_exc(limit=limit), output=file)
 
 
@@ -578,7 +619,7 @@ class DeferredSignal(object):
     priority and prevents multiple calls from being inserted in the
     mainloop at a time, greatly improving responsiveness in some places.
 
-    When the target function will finally be called the arguments passed
+    When the target function is finally called, the arguments passed
     are the last arguments passed to DeferredSignal.
 
     `priority` defaults to GLib.PRIORITY_DEFAULT
@@ -617,8 +658,8 @@ class DeferredSignal(object):
                 timeout, f, priority=priority)
 
     @property
-    def im_self(self):
-        return self.func.im_self
+    def __self__(self):
+        return self.func.__self__
 
     @property
     def __code__(self):
@@ -682,8 +723,8 @@ def _connect_destroy(sender, func, detailed_signal, handler, *args, **kwargs):
     to the bound method and the bound to object doesn't get GCed.
     """
 
-    if hasattr(handler, "im_self"):
-        obj = handler.im_self
+    if hasattr(handler, "__self__"):
+        obj = handler.__self__
     else:
         # XXX: get the "self" var of the enclosing scope.
         # Used for nested functions which ref the object but aren't methods.
@@ -736,7 +777,7 @@ def sanitize_tags(tags, stream=False):
     """
 
     san = {}
-    for key, value in tags.iteritems():
+    for key, value in iteritems(tags):
         key = key.lower()
         key = {"location": "website"}.get(key, key)
 
@@ -875,56 +916,16 @@ def gi_require_versions(name, versions):
 
     import gi
 
+    error = None
     for version in versions:
         try:
             gi.require_version(name, version)
         except ValueError as e:
-            pass
+            error = e
         else:
             return version
     else:
-        raise e
-
-
-@contextlib.contextmanager
-def atomic_save(filename, suffix, mode):
-    """Try to replace the content of a file in the safest way possible.
-
-    * filename+suffix will be created during the process.
-    * On UNIX this operation is atomic, on Windows it is not.
-
-    with atomic_save("config.cfg", ".tmp", "wb") as f:
-        f.write(data)
-
-    Can raise.
-    """
-
-    assert is_fsnative(filename)
-
-    temp_filename = filename + suffix
-    fileobj = open(temp_filename, "wb")
-    try:
-        if fcntl is not None:
-            fcntl.flock(fileobj.fileno(), fcntl.LOCK_EX)
-
-        yield fileobj
-
-        fileobj.flush()
-        os.fsync(fileobj.fileno())
-
-        # No atomic rename on windows
-        if os.name == "nt":
-            fileobj.close()
-            try:
-                os.remove(filename)
-            except EnvironmentError:
-                pass
-
-        os.rename(temp_filename, filename)
-    finally:
-        if fcntl is not None:
-            fcntl.flock(fileobj.fileno(), fcntl.LOCK_UN)
-        fileobj.close()
+        raise error
 
 
 def load_library(names, shared=True):
@@ -946,10 +947,23 @@ def load_library(names, shared=True):
     else:
         load_func = ctypes.cdll.LoadLibrary
 
+    if is_osx():
+        # make sure it's either empty or contains /usr/lib.
+        # (jhbuild sets it for example). Otherwise ctypes can't
+        # find libc (bug?)
+        if "DYLD_FALLBACK_LIBRARY_PATH" in os.environ:
+            paths = os.environ["DYLD_FALLBACK_LIBRARY_PATH"]
+            paths = paths.split(os.pathsep)
+            if "/usr/lib" not in paths:
+                paths.append("/usr/lib")
+                os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = \
+                    os.pathsep.join(paths)
+
     errors = []
     for name in names:
         dlopen_name = name
-        if ".so" not in name and ".dll" not in name:
+        if ".so" not in name and ".dll" not in name and \
+                ".dylib" not in name:
             dlopen_name = ctypes.util.find_library(name) or name
 
         try:
@@ -1094,41 +1108,6 @@ def re_escape(string, BAD="/.^$*+-?{,\\[]|()<>#=!:"):
     return type(string)().join(map(needs_escape, string))
 
 
-def enum(cls):
-    """Class decorator for enum types::
-
-        @enum
-        class SomeEnum(object):
-            FOO = 0
-            BAR = 1
-
-    Result is an int subclass and all attributes are instances of it.
-    """
-
-    assert cls.__bases__ == (object,)
-
-    d = dict(cls.__dict__)
-    new_type = type(cls.__name__, (int,), d)
-    new_type.__module__ = cls.__module__
-
-    map_ = {}
-    for key, value in d.iteritems():
-        if key.upper() == key and isinstance(value, (int, long)):
-            value_instance = new_type(value)
-            setattr(new_type, key, value_instance)
-            map_[value] = key
-
-    def repr_(self):
-        if self in map_:
-            return "%s.%s" % (type(self).__name__, map_[self])
-        else:
-            return "%s(%s)" % (type(self).__name__, self)
-
-    setattr(new_type, "__repr__", repr_)
-
-    return new_type
-
-
 def set_process_title(title):
     """Sets process name as visible in ps or top. Requires ctypes libc
     and is almost certainly *nix-only. See issue 736
@@ -1162,31 +1141,11 @@ def list_unique(sequence):
     return l
 
 
-def set_win32_unicode_argv():
-    if os.name != "nt":
-        return
+def reraise(tp, value, tb=None):
+    """Reraise an exception with a new exception type and
+    the original stack trace
+    """
 
-    import ctypes
-    from ctypes import cdll, windll, wintypes
-
-    GetCommandLineW = cdll.kernel32.GetCommandLineW
-    GetCommandLineW.argtypes = []
-    GetCommandLineW.restype = wintypes.LPCWSTR
-
-    CommandLineToArgvW = windll.shell32.CommandLineToArgvW
-    CommandLineToArgvW.argtypes = [
-        wintypes.LPCWSTR, ctypes.POINTER(ctypes.c_int)]
-    CommandLineToArgvW.restype = ctypes.POINTER(wintypes.LPWSTR)
-
-    LocalFree = windll.kernel32.LocalFree
-    LocalFree.argtypes = [wintypes.HLOCAL]
-    LocalFree.restype = wintypes.HLOCAL
-
-    argc = ctypes.c_int()
-    argv = CommandLineToArgvW(GetCommandLineW(), ctypes.byref(argc))
-    if not argv:
-        return
-
-    sys.argv = argv[max(0, argc.value - len(sys.argv)):argc.value]
-
-    LocalFree(argv)
+    if tb is None:
+        tb = sys.exc_info()[2]
+    py_reraise(tp, value, tb)
